#!/usr/bin/perl
use strict;

### Input file format (mpileup):
#   chr1    14643   G       177     ,$,$,$,$,$,$,$....
### Output file format (extended sites); for -, letters read like the RNA:
#   chr1	15015	C	G	-	80	27	0.747

my %Reverse;
$Reverse{"A"} = "T";
$Reverse{"C"} = "G";
$Reverse{"G"} = "C";
$Reverse{"T"} = "A";

print "#chrom\tpos\trefbase_RNA\tmutbase_RNA\tstrand\trefbase_cov\tmutbase_cov\tinclusion_ratio\n";
while (<>) {

  /^(\S+)\t(\d+)\t(\S)\t(\d+)\t(\S+)\t\S+\t(\S)$/ or die "illegal line. $_";
  my ($chrom,$pos,$refbase,$numreads,$match_list,$sign) = ($1,$2,$3,$4,$5,$6);

  next if ($sign eq "*");
# next if ($sign eq ".");  # no annotation file given, keep everything

  ($sign eq "+") or ($sign eq "-") or ($sign eq ".") or die "Unrecognized sign/orientation. $_";

  my %Count;

  my $match_count = ($match_list =~ tr/\.,/\.,/);
  $Count{"A"} = ($match_list =~ tr/Aa/Aa/);
  $Count{"C"} = ($match_list =~ tr/Cc/Cc/);
  $Count{"G"} = ($match_list =~ tr/Gg/Gg/);
  $Count{"T"} = ($match_list =~ tr/Tt/Tt/);

  if ($refbase eq 'A') { $Count{"A"} = $match_count; } 
  if ($refbase eq 'C') { $Count{"C"} = $match_count; }
  if ($refbase eq 'G') { $Count{"G"} = $match_count; }
  if ($refbase eq 'T') { $Count{"T"} = $match_count; }
  
  my $crtbase;
  if (!($refbase eq "A") && $Count{"A"}) { $crtbase = "A"; }
  if (!($refbase eq "C") && $Count{"C"}) { $crtbase = "C"; }
  if (!($refbase eq "G") && $Count{"G"}) { $crtbase = "G"; }
  if (!($refbase eq "T") && $Count{"T"}) { $crtbase = "T"; }

  my $t;
  if ($sign ne "-") {
    print "$chrom\t$pos\t$refbase\t$crtbase\t$sign\t", $Count{$refbase}, "\t", ($t=($Count{"A"}+$Count{"C"}+$Count{"G"}+$Count{"T"}-$Count{$refbase}));
    printf "\t%1.3f\n", ($t/($Count{"A"}+$Count{"C"}+$Count{"G"}+$Count{"T"})), "\n";
  } else {
    my $rev_refbase = $Reverse{$refbase};
    my $rev_crtbase = $Reverse{$crtbase};
    print "$chrom\t$pos\t$rev_refbase\t$rev_crtbase\t$sign\t", $Count{$refbase}, "\t", ($t=($Count{"A"}+$Count{"C"}+$Count{"G"}+$Count{"T"}-$Count{$refbase}));
    printf "\t%1.3f\n", ($t/($Count{"A"}+$Count{"C"}+$Count{"G"}+$Count{"T"})), "\n";
  }
}
