#include "libc.h"
#include "lib.h"

#include "fasta-c.h"

#define DEFAULT_ALLOC 10000

static char Usage[] = "%s sites annot genome\n";

#ifdef MOUSE
char *Chrom[] = {"chr1", "chr2", "chr3", "chr4","chr5",
                 "chr6", "chr7", "chr8", "chr9", "chr10",
                 "chr11", "chr12", "chr13", "chr14","chr15",
                 "chr16", "chr17", "chr18", "chr19", "chrX",
                 "chrY", "chrM"};
int numChroms = 22;
#else
char *Chrom[] = {"chr1", "chr2", "chr3", "chr4","chr5",
                 "chr6", "chr7", "chr8", "chr9", "chr10",
                 "chr11", "chr12", "chr13", "chr14","chr15",
                 "chr16", "chr17", "chr18", "chr19", "chr20",
                 "chr21", "chr22", "chrX", "chrY", "chrM"};
int numChroms = 25;
#endif

typedef struct {
   int   gaid;
   int   from, to;
   char  ori;
} exon_t;

typedef struct {
   int  pos, gaid;
   char ref, mut;
} mutation_t;


static char   errmsg[100];

static exon_t **Exons;
static int     *exonCounts;
static int      numExons, maxExons;
static int      N;
static void    *xFastA;

static void loadExons(const char *filename, exon_t **exons, int *numexons, int *maxexons);
static int  lookup_chrom(char *subj, char *chroms[], int numchroms);
static int  exon_cmp(const void *a, const void *b);
static int  lookup_exon(int pos, int igaidgaidgaiddx, exon_t *exons, int start, int numexons);

static int  exon_cmp(const void *a, const void *b) {
   exon_t *A = (exon_t *)a;
   exon_t *B = (exon_t *)b;
   int t;

   if (t=A->gaid-B->gaid)
     return t;
   else if (t=A->from-B->from) 
     return t;
   else
     return (A->to-B->to);
}

void main(int argc, char *argv[])
{
    mutation_t   mut;
    char         buf[100000], dummy[100], match_list[100000];
    char        *seq, *s;
    int          idxc, idxi, idxx, i, j, k;
    int          crtGAid = -1;
    char        *crtScaff = NULL;
    FILE        *fp;
  
    if (argc<4) fatalf(Usage, argv[0]);

    xFastA = createFastA(argv[3]);
    
    numExons = maxExons = 0;

    Exons = (exon_t **)malloc(numChroms*sizeof(exon_t));
    memset(Exons, 0, numChroms*sizeof(exon_t *));

    loadExons(argv[2], &(Exons[0]), &numExons, &maxExons);

    qsort(Exons[0], numExons, sizeof(exon_t), exon_cmp); 

    exonCounts = (int *)malloc(numChroms*sizeof(int));
    memset(exonCounts, 0, numChroms*sizeof(int));

    for (i=0; i<numExons; i++) {
      int gaid = Exons[0][i].gaid;
      if (Exons[gaid]==NULL) Exons[gaid] = Exons[0]+i; 
      exonCounts[gaid]++;
    }

    fp = ckopen(argv[1], "r");
    while (fgets(buf, 100000, fp)!=NULL) {
       int idummy; 
       /* chr1    1250898 A       34      ,,,c,,,,..,,,,c,cccc,cccc,cccccc,c      HH>#FHFHHHDH5FHCHF#HHFHGHIFHHHDHGD */
       if (sscanf(buf, "%s\t%d\t%c\t%d\t%s\t",
                  dummy,&(mut.pos),&(mut.ref),&idummy,match_list)!=5) {
         fprintf(stderr, "Unrecognized mutation line. Skipping site. %s\n", buf);
         continue;
       }

       if ((idxc=lookup_chrom(dummy, Chrom, numChroms))<0) {
         fprintf(stderr, "Unrecognized chromosome number. Skipping site. %s\n", buf);
         continue;
       }

       for (s=match_list; *s; s++) {
         char ch = toupper(*s);
         if (ch=='A' || ch=='C' || ch=='G' || ch=='T') {
            mut.mut = ch; break;
         }
       }

       if ((idxi=lookup_exon(mut.pos, idxc, Exons[idxc], 0, exonCounts[idxc]))>=0) {
         exon_t  *xptr = Exons[idxc]+idxi;
         int start, len, end, offset;

         if (crtGAid!=idxc) {
           assert(xptr->gaid == idxc);
           crtScaff = getFastAsequence(xFastA,xptr->gaid);
           crtGAid = idxc;
         } 

         seq = crtScaff+xptr->from-1;

         if (mut.pos-xptr->from <= xptr->to-mut.pos) {
#if 0
           /* length < 100 even if exon > 100 */
           start = max(mut.pos-xptr->from-50, 0);
           len = min(mut.pos-xptr->from,50) + 1 + min(xptr->to-mut.pos,50);
           end = start + len-1;
           offset = mut.pos-xptr->from-start;
#else
           /* printf ("(1)\n"); */
           start = max(mut.pos-xptr->from-50, 0);
           end = min(start+100,xptr->to-xptr->from);
           len = end-start+1;
           offset = mut.pos-xptr->from-start;
#endif
         } else {
#if 0
           /* length < 100 even if exon > 100 */
           end = (xptr->to-xptr->from) - max(xptr->to-mut.pos-50,0);
           len = min(xptr->to-mut.pos,50) + 1 + min(mut.pos-xptr->from,50);
           start = end-(len-1);
           offset = mut.pos-xptr->from-start;
#else
           /* printf ("(2)\n"); */
           end = (xptr->to-xptr->from) - max(xptr->to-mut.pos-50,0);
           start = max(0,end-100);
           len = end-start+1;
           offset = mut.pos-xptr->from-start;
#endif
         }

         printf(">%s:%d:%c:%c %d (%d)\n", dummy, mut.pos, mut.ref, mut.mut, offset, len);
         for (s=seq+start; s<seq+start+len; s++) 
           printf("%c", *s);
         printf("\n");
       } else {
         fprintf(stderr, "Warning:\tNo exon found for %s. Skipping.\n", buf);
       }
    }
    fclose(fp);

    free(Exons[0]);
    free(Exons);

    destroyFastA(xFastA);

    exit(0);
}


static void loadExons(const char *filename, exon_t **exons, int *numexons, int *maxexons)
{
   char *hdr, buf[1000000], *s, *q, ori, chrom[20], dummy[5], dummy1[50];
   int   idx, from, to, i;
   FILE *fp;

  *numexons = *maxexons = idx = 0;

   /* read in an annotation file (gtf), make a list of exons (may contain duplicates) */
   fp = ckopen(filename, "r");
    
   while (fgets(buf,1000000,fp)!=NULL) {

     if (!strstr(buf, "\texon\t")) continue; 
      
     if (sscanf(buf, "%s\t%s\texon\t%d\t%d\t%s\t%c\t", chrom, dummy, &from, &to, dummy1, &ori)!=6) {
       fprintf(stderr, "Warning: Unrecognized exon line in gtf file. Skipping. %s\n", buf);
       continue;
     }
     if ((i=lookup_chrom(chrom, Chrom, numChroms))<0) {
       fprintf(stderr, "Warning: Cannot find chromosome. Skipping exon. %s\n", chrom); 
       continue;
     }

     if (*numexons>=*maxexons) {
       *maxexons = max(DEFAULT_ALLOC, 2*(*maxexons));
       *exons = realloc(*exons, (*maxexons)*sizeof(exon_t));
     }

     (*exons)[*numexons].from = from;
     (*exons)[*numexons].to = to; 
     (*exons)[*numexons].gaid = i;
     (*exons)[*numexons].ori = ori;

     (*numexons)++;
   }
}

static int lookup_chrom(char *subj, char *chroms[], int numchroms)
{
   int i;

   for (i=0; (i<numchroms) && strcmp(chroms[i],subj); i++);
   return (i==numchroms) ? -1 : i;
}

static int lookup_exon(int pos, int gaid, exon_t *exons, int start, int numexons)
{
   int i, j;
 
   for (i=start; (i<numexons) && (pos>=exons[i].from); i++) {
     if (exons[i].gaid!=gaid) {
       sprintf(errmsg, "Incorrect gaid in lookup list. Found %d, expecting %d.\n", exons[i].gaid, gaid);
       fatalf("%s", errmsg);
     }
     if (exons[i].from<=pos && pos<=exons[i].to) return i;
   }
     
   return -1;
}
