#!/usr/bin/perl
use strict;

($#ARGV==1) or die "Usage: $0 minreads pctreads < mpileup\n"; 

my $R = shift;
my $P = shift;

# Input format: 
# chr1    14643   G       177     ,$,$,$,$,$,$,$....


while (<>) {

  my ($refbase,$numreads,$match_list);

  if (/^\S+\t\d+\t(\S)\t(\d+)\t(\S+)\t/) {
    ($refbase,$numreads,$match_list) = ($1,$2,$3);
  } else {
     die "illegal line in input file. $_";
  }
  next if ($numreads<$R);

  my $match_count = ($match_list =~ tr/\.,/\.,/);
  next if ($match_count == $numreads);

  next if ($match_list =~ /[\-+]/); # skip variable sites that are also indel sites

  my $A_count = ($match_list =~ tr/Aa/Aa/);
  my $C_count = ($match_list =~ tr/Cc/Cc/);
  my $G_count = ($match_list =~ tr/Gg/Gg/);
  my $T_count = ($match_list =~ tr/Tt/Tt/);

  if ($refbase eq 'A') { $A_count = $match_count; } 
  if ($refbase eq 'C') { $C_count = $match_count; }
  if ($refbase eq 'G') { $G_count = $match_count; }
  if ($refbase eq 'T') { $T_count = $match_count; }
  
  my $n0 = 0;
  $A_count or $n0++;
  $C_count or $n0++;
  $G_count or $n0++;
  $T_count or $n0++;

  next if ($n0!=2);
  next if (!$match_count);  # if none of the letters is the reference base

  if ((!$A_count || (100*$A_count>=$P*$numreads)) &&
      (!$C_count || (100*$C_count>=$P*$numreads)) &&
      (!$G_count || (100*$G_count>=$P*$numreads)) &&
      (!$T_count || (100*$T_count>=$P*$numreads))) {
      print $_;
  }
}
