#!/usr/bin/perl
use strict;

($#ARGV>=0) or die "Usage: $0 (a-var)sites [-v] < tmp(a-var)sites\n";

my $sitesFile = shift;
my $verbose = shift;
!defined($verbose) or ($verbose eq "-v") or die "Usage: $0 (a-var)sites [-v] < tmp(a-var)sites\n";
  

my %Reverse;

$Reverse{"a"} = $Reverse{"A"} = "T";
$Reverse{"c"} = $Reverse{"C"} = "G";
$Reverse{"g"} = $Reverse{"G"} = "C";
$Reverse{"t"} = $Reverse{"T"} = "A";
$Reverse{"n"} = $Reverse{"N"} = "N";

my %Pass;
open(F, "<$sitesFile") or die "$sitesFile.\n";
### determine whether this is a final-sites or a pileup file
my $isPileup;
while (<F>) {
   next if /^#/;
  
   if (!defined($isPileup)) {
     if (/^\S+\t\d+\t\S\t\d+\t/) {
       $isPileup = 1;
     } elsif (/^\S+\t\d+\t\S\t\S\t\S$/) {
       $isPileup = 0; 
     } else {
       die "illegal line. (0). $_";
     }
   }

   my ($chrom,$pos,$ref,$mut,$sign,$match_list);
   if ($isPileup == 0) {
     /^(\S+)\t(\d+)\t(\S)\t(\S)\t(\S)/ or die "illegal line (1). $_";
     ($chrom,$pos,$ref,$mut,$sign) = ($1,$2,$3,$4,$5);

     # convert to genomic letters
     if ($sign == '-') {
       $mut = $Reverse{$mut};
       $ref = $Reverse{$ref};
     }
   } else {
     if (/^(\S+)\t(\d+)\t(\S)\t\d+\t(\S+)\t/) {
       ($chrom,$pos,$ref,$match_list) = ($1,$2,$3,$4);

       my $mut;
       if ($match_list =~ /[Aa]/) { $mut = "A"; }
       elsif ($match_list =~ /[Cc]/) { $mut = "C"; }
       elsif ($match_list =~ /[Gg]/) { $mut = "G"; }
       elsif ($match_list =~ /[Tt]/) { $mut = "T"; }
     }
   }
   $Pass{"$chrom:$pos:$ref:$mut"} = 1;
#  print "PASS: $chrom $pos $ref $mut\n";
}
close(F);

my $count = 0;
while (<>) {
  my ($chrom,$pos,$refbase,$mutbase,$match_list,$sign);
  if (/^(\S+)\t(\d+)\t(\S)\t\d+\t(\S+)\t/) {
    ($chrom,$pos,$refbase,$match_list) = ($1,$2,$3,$4);

    my $mutbase;
    if ($match_list =~ /[Aa]/) { $mutbase = "A"; }
    elsif ($match_list =~ /[Cc]/) { $mutbase = "C"; }
    elsif ($match_list =~ /[Gg]/) { $mutbase = "G"; }
    elsif ($match_list =~ /[Tt]/) { $mutbase = "T"; }

  } elsif (/^\S\t(\S+)\t(\d+)\t(\S)\t(\S)\t/) {
    ($chrom,$pos,$refbase,$mutbase,$sign) = ($1,$2,3,$4,$5);

    # convert to genomic letters
    if ($sign == '-') {
      $mutbase = $Reverse{$mutbase};
      $refbase = $Reverse{$refbase};
    }
  }

# print "CHECK: $chrom $pos $refbase $mutbase\n";
  next if !defined($Pass{"$chrom:$pos:$refbase:$mutbase"});
# print "YES: $chrom $pos $refbase $mutbase\n";

  $count++; 
  if (defined($verbose)) { print $_; }
}
defined($verbose) or print $count . "\n";
