#include "fasta.H"

#include <stdio.h>
#include <string.h>

/* fasta-find-id.C - Library of routines for random access to sequences
   in a multi-fasta file. 
   Written by: Brian Walenz

   This file is part of the AIR package distributed under the GNU license,
   and containing portions copyright of:
        Copyright (C) 2004  Liliana Florea 
        Copyright (C) 2004  Brian Walenz
        Copyright (C) 1996  Xiaoqiu Huang

   AIR is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

char*
FastAWrapper::moveToNextName(char *ptr, u32bit iid) {

  //  XXX:
  //
  //  Sadly, if our index has saved only the id, we don't know the
  //  length of this field.  We must blindly search for the start of
  //  the next id.
  //
  //  If we have the whole defline, then we know the length.
  //
  if (_theGlobalDesc._indexType & FASTA_INDEX_MASK == FASTA_INDEX_PLUS_IDS) {
    while (*ptr)
      ptr++;
    ptr++;
  } else {
    ptr += _theSeqs[iid]._headerLen + 1;
  }

  return(ptr);
}



bool
FastAWrapper::find(char *id) {
  u32bit         iid = 0;
  char          *ptr = _theNames;
  char          *res = 0L;

  //
  //  need to trim off the whitespace from the front/end of the id
  //

  //  This is just too ugly.
  //
#if 0
  u32bit         len = 0;

  char *tmpid = new char [ strlen(id) + 1];
  while (*id && isspace(*id))
    id++;

  for (len=0; *id; len++, id++)
    tmpid[len] = *id;

  if (len == 0) {
    delete [] tmpid;
    return(false);
  }

  while (isspace(tmpid[len]))
    len--;

  len++;
  tmpid[len] = 0;

  id = tmpid;
#endif

  while (iid < _theGlobalDesc._numberOfSequences) {
    res = strstr(ptr, id);
    if (res) {
      _filebuffer->seek(_theSeqs[iid]._headerStart);
      _currentSequenceNumber = iid;
      return(true);
    }

    ptr = moveToNextName(ptr, iid);

    iid++;
  }
    
  return(false);
}



