#include "fasta.H"

#include <stdio.h>
#include <string.h>

/* fasta-find-iid.C - Library of routines for random access to sequences in
   a multi-fasta file.
   Written by: Brian Walenz

   This file is part of the AIR package distributed under the GNU license,
   and containing portions copyright of:
        Copyright (C) 2004  Liliana Florea 
        Copyright (C) 2004  Brian Walenz
        Copyright (C) 1996  Xiaoqiu Huang

   AIR is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


bool
FastAWrapper::find(IID_t  iid) {

  if (!_isRandomAccess) {
    fprintf(stderr, "FastAWrapper::find(IID)-- ERROR: '%s' is not open for random access; find() failed.\n", _filename);
    return(false);
  }

  if (iid >= _theGlobalDesc._numberOfSequences) {
    fprintf(stderr, "FastAWrapper::find(IID)-- ERROR: index of "u32bitFMT" too large for '%s' (only "u32bitFMT" sequences).\n",
            iid, _filename, _theGlobalDesc._numberOfSequences);
    return(false);
  }

  if (iid != _currentSequenceNumber) {
    _filebuffer->seek(_theSeqs[iid]._headerStart);
    _currentSequenceNumber = iid;
  }

  return(true);
}
