#ifndef FASTA_TYPES_H
#define FASTA_TYPES_H


#ifndef BRI_H
#define BRI_H

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#if defined(__APPLE__) || defined(__FreeBSD__) || defined(__CYGWIN__)
#include <sys/syslimits.h>
#endif

#if defined(__alpha) || defined(_AIX)
#define TRUE64BIT
#endif

#ifdef TRUE64BIT

//  Generic 64-bit platform

typedef unsigned long       u64bit;
typedef unsigned int        u32bit;
typedef signed long         s64bit;
typedef signed int          s32bit;
typedef unsigned short      u16bit;
typedef signed short        s16bit;
typedef unsigned char       u8bit;
typedef signed char         s8bit;

#define  u64bitZERO      (0x0000000000000000LU)
#define  u64bitONE       (0x0000000000000001LU)
#define  u64bitMAX       (0xffffffffffffffffLU)
#define  u64bitMASK(X)   ((~u64bitZERO) >> (64 - (X)))
#define  u64bitFMTW(X)   "%" #X "lu"
#define  u64bitFMT       "%lu"
#define  u64bitHEX       "0x%016lx"
#define  s64bitFMTW(X)   "%" #X "ld"
#define  s64bitFMT       "%ld"

#define  u32bitZERO      (0x00000000U)
#define  u32bitONE       (0x00000001U)
#define  u32bitMAX       (0xffffffffU)
#define  u32bitMASK(X)   ((~u32bitZERO) >> (32 - (X)))
#define  u32bitFMTW(X)   "%" #X "u"
#define  u32bitFMT       "%u"
#define  u32bitHEX       "0x%08x"
#define  s32bitFMTW(X)   "%" #X "d"
#define  s32bitFMT       "%d"

#define  u16bitZERO      (0x0000)
#define  u16bitONE       (0x0001)
#define  u16bitMAX       (0xffff)
#define  u16bitMASK(X)   ((~u16bitZERO) >> (16 - (X)))

#define  u8bitZERO       (0x00)
#define  u8bitONE        (0x01)
#define  u8bitMAX        (0xff)
#define  u8bitMASK(X)    ((~u8bitZERO) >> (8 - (X)))

//  UNTESTED!
#define  strtou32bit(N,O) strtoul(N, O, 0)
#define  strtou64bit(N,O) strtoul(N, O, 0)

#else

//  Generic 32-bit platform

typedef unsigned long long  u64bit;
typedef unsigned long       u32bit;
typedef signed long long    s64bit;
typedef signed long         s32bit;
typedef unsigned short      u16bit;
typedef signed short        s16bit;
typedef unsigned char       u8bit;
typedef signed char         s8bit;

#define  u64bitZERO      (0x0000000000000000LLU)
#define  u64bitONE       (0x0000000000000001LLU)
#define  u64bitMAX       (0xffffffffffffffffLLU)
#define  u64bitMASK(X)   ((~u64bitZERO) >> (64 - (X)))
#define  u64bitFMTW(X)   "%" #X "llu"
#define  u64bitFMT       "%llu"
#define  u64bitHEX       "0x%016llx"
#define  s64bitFMTW(X)   "%" #X "lld"
#define  s64bitFMT       "%lld"

#define  u32bitZERO      (0x00000000LU)
#define  u32bitONE       (0x00000001LU)
#define  u32bitMAX       (0xffffffffLU)
#define  u32bitMASK(X)   ((~u32bitZERO) >> (32 - (X)))
#define  u32bitFMTW(X)   "%" #X "lu"
#define  u32bitFMT       "%lu"
#define  u32bitHEX       "0x%08lx"
#define  s32bitFMTW(X)   "%" #X "ld"
#define  s32bitFMT       "%ld"

#define  u16bitZERO      (0x0000)
#define  u16bitONE       (0x0001)
#define  u16bitMAX       (0xffff)
#define  u16bitMASK(X)   ((~u16bitZERO) >> (16 - (X)))

#define  u8bitZERO       (0x00)
#define  u8bitONE        (0x01)
#define  u8bitMAX        (0xff)
#define  u8bitMASK(X)    ((~u8bitZERO) >> (8 - (X)))

#define  strtou32bit(N,O) strtoul(N, O, 0)
#define  strtou64bit(N,O) strtoull(N, O, 0)

#endif  //  TRUE64BIT



#ifdef __cplusplus
extern "C" {
#endif

//  Create the O_LARGEFILE type for open(), if it doesn't already
//  exist (FreeBSD, Tru64).  We assume that by including the stuff
//  needed for open(2) we'll get any definition of O_LARGEFILE.
//
#ifndef O_LARGEFILE
#define O_LARGEFILE    0
#endif

void*
mapFile(const char *filename,
        size_t     *length,
        char        mode);

void
unmapFile(void     *addr,
          size_t    length);

#ifdef __cplusplus
}
#endif

#endif  //  BRI_H






//  Types used in the index
//
typedef off_t  _idxfa_pos;
typedef u32bit _idxfa_len;

//  INDEX TYPES
//
//  Types are currently subsets.  An INDEX_ONLY is a PLUS_IDS is a
//  PLUS_DEFLINES This is used during the openIndex().
//
#define FASTA_INDEX_ANY                0x00000000

#define FASTA_INDEX_ONLY               0x00000001
#define FASTA_INDEX_PLUS_IDS           0x00000002
#define FASTA_INDEX_PLUS_DEFLINES      0x00000003

#define FASTA_INDEX_MASK               0x000000ff

#define FASTA_INDEX_MD5                0x80000000


//  FASTA TYPES
//
#define FASTA_UNDECLARED    0x00000001


struct _idxfa_global {
  u64bit       _magic;
  u32bit       _version;
  u32bit       _indexType;
  u32bit       _fastaType;
  u32bit       _numberOfSequences;      //  Number of sequences in the file
  off_t        _fastaFileSize;          //  st_size  - size of file in bytes
  time_t       _fastaModificationTime;  //  st_mtime - time of last data modification
  time_t       _fastaCreationTime;      //  st_ctime - time of last file status change
  _idxfa_len   _seqlineLength;          //  Length of the sequence lines, if fixed width
  _idxfa_len   _seqendlLength;          //  Length of the sequence line terminators, if fixed width (really, "\n" or "\r\n"?)
  bool         _fixedWidth;             //  All (but last) lines are the same width, and there is no embedded space
  bool         _squeezedSequences;      //  No whitespace in the sequences
  char         _alphabet[256];          //  true/false for each character seen
};

//  20040508 - On the next major change, this structure needs to be
//  rearranged so that 8-byte values are 8-byte aligned (moving
//  _seqStart before _headerLen will do that).  This will fix an
//  optimization on OS-X that tries to align data to their natural
//  boundaries.
//
#ifdef __APPLE__

//  This is the "correct" version that we should use everywhere.
//
struct _idxfa_desc {
  _idxfa_pos   _headerStart;       //  Position of the header in the file
  _idxfa_pos   _seqStart;          //  Position of the sequence in the file
  _idxfa_len   _headerLen;         //  Length of the header, not counting the '\n'
  _idxfa_len   _seqLen;            //  Length of the sequence
};

#else

//  Ick!
//
struct _idxfa_desc {
  _idxfa_pos   _headerStart;       //  Position of the header in the file
  _idxfa_len   _headerLen;         //  Length of the header, not counting the '\n'
  _idxfa_pos   _seqStart;          //  Position of the sequence in the file
  _idxfa_len   _seqLen;            //  Length of the sequence
};

#endif


#endif  //  FASTA_TYPES_H
