#!/usr/bin/perl
use strict;

### Can take as input either a mpileup-style or a sites-style file

($#ARGV>=2) or die "Usage: $0 mpileup_sites snps [-i ignore-src1,ignore-src2,...] [-p prefix]\n";
my $sitesFile = shift;
my $snpsFile = shift;
my $prefix;
my @ignoreSets = ();

my %Ignore;
my $arg;
while ($#ARGV >= 0) {
  $arg = shift;
  if ($arg eq "-i") {
    $arg = shift;
    @ignoreSets = split ',', $arg;
    foreach my $k (@ignoreSets) {
      $Ignore{$k} = 1;
    }
  } elsif ($arg eq "-p") {
    $prefix = shift;
  } else {
    die "Usage: $0 mpileup_sites snps [-i ignore-src1,ignore-src2,...] [-p prefix]\n";
  }
}


### Codes: 0 - RDD or unassigned; 1 - SNP; 2 - high-confidence (control) SNP

my %Sites;

open(S, "<$sitesFile") or die "could not open file $sitesFile.\n";
while (<S>) {
  # if pileup file
  
  my ($chrom,$pos,$refbase,$mutbase,$match_list);
  if (/^(\S+)\t(\d+)\t(\S)\t\d+\t(\S+)\t/) {
    ($chrom,$pos,$refbase,$match_list) = ($1,$2,$3,$4);

    if ($match_list =~ /[Aa]/) { $mutbase = "A"; }
    elsif ($match_list =~ /[Cc]/) { $mutbase = "C"; }
    elsif ($match_list =~ /[Gg]/) { $mutbase = "G"; }
    elsif ($match_list =~ /[Tt]/) { $mutbase = "T"; }

    $Sites{"$chrom:$pos"} = 0;
    $Sites{"$chrom:$pos:$refbase:$mutbase"} = 0;
  } elsif (/^\S\t(\S+)\t(\d+)\t(\S)\t(\S)\t/) {
    ($chrom,$pos,$refbase,$mutbase) = ($1,$2,3,$4);

    $Sites{"$chrom:$pos"} = 0;
    $Sites{"$chrom:$pos:$refbase:$mutbase"} = 0;
  } else {
    die "illegal line in input sites file. $_";
  }
}
close(S);

my $count = 0;
open(F, "<$snpsFile") or die "could not open SNPs file $snpsFile.\n";
while (<F>) {
  chomp;

  if (($count % 1000000)==0) {
    print STDERR "Read $count SNPs\r";
  }

  /^(\S+)\s(\d+)\s(\S)\s(\S)\s(\S+)$/ or die "illegal vcf line. $_";
  my ($chrom,$pos,$ref,$mut,$src) = ($1,$2,$3,$4,$5);

  $count++;

  next if !defined($Sites{"$chrom:$pos"});
  if ($Sites{"$chrom:$pos"} == 0) {
    $Sites{"$chrom:$pos"} == 1;
  }

  if (!defined($Ignore{$src}) && defined($Sites{"$chrom:$pos:$ref:$mut"})) {
    $Sites{"$chrom:$pos:$ref:$mut"} = 2;
    $Sites{"$chrom:$pos"} = 2;
  }
}
print STDERR "Finished reading SNPs.\n";
close(F);

open(R, ">$prefix.RDD") or die "could not open file $prefix.RDD for writing.\n";
open(P, ">$prefix.SNP") or die "could not open file $prefix.SNP for writing.\n";

my $countRDD = 0;
my $countSNP = 0;
open(S, "<$sitesFile") or die "could not open file $sitesFile.\n";
while (<S>) {
  # if pileup file
 
  my ($chrom,$pos,$refbase,$mutbase,$match_list);
  if (/^(\S+)\t(\d+)\t(\S)\t\d+\t(\S+)\t/) {
    ($chrom,$pos,$refbase,$match_list) = ($1,$2,$3,$4);

    # Reset mutbase to the actual variant letter
    if ($match_list =~ /[Aa]/) { $mutbase = "A"; }
    elsif ($match_list =~ /[Cc]/) { $mutbase = "C"; }
    elsif ($match_list =~ /[Gg]/) { $mutbase = "G"; }
    elsif ($match_list =~ /[Tt]/) { $mutbase = "T"; }
  } elsif (/^\S\t(\S+)\t(\d+)\t(\S)\t(\S)\t/) {
    ($chrom,$pos,$refbase,$mutbase) = ($1,$2,3,$4);
  }

  if ($Sites{"$chrom:$pos"} == 0) {
    print R $_;
    $countRDD++;
  } elsif ($Sites{"$chrom:$pos:$refbase:$mutbase"} == 2) {
    print P $_;
    $countSNP++;
  }
}
close(S);

close(R);
close(P);
